/*
 * bench_in/bench_in.c -- FX2 USB bulk USB input firmware for host
 *						  controller benchmarking.
 *
 * Copyright (c) 2006--2008 by Wolfgang Wieser ] wwieser (a) gmx <*> de [
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation. (See COPYING.GPL for details.)
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

//------------------------------------------------------------------------------
// NOTE: In order for this test to work, the following manual wire connections
//		 have to be done on the USB-FX2 board:
//	 PA2/SLOE	   <--> +3V3
//	 PA4/FIFOADR0  <-->  GND
//	 PA5/FIFOADR1  <--> +3V3
//	 RDY0/SLRD	   <--> +3V3
//	 RDY1/SLWR	   <-->  SAMPLING_CLK
//------------------------------------------------------------------------------

#define ALLOCATE_EXTERN
#include <fx2regs.h>


// Read TRM p.15-115 for an explanation on this.
// A single nop is sufficient for default setup but like that we're on
// the safe side.
#define	NOP		_asm nop _endasm
#define	SYNCDELAY	NOP; NOP; NOP; NOP


// Initialize the FX2 in 16bit sync fifo mode.
static void Initialize(void)
{
	//CPUCS: 0 0 PORTCSTB CLKSPD1	CLKSPD0 CLKINV CLKOE 8051RES   00000010
	//bit5	 PORTCSTB=1: reads/writes to PORTC generate RD# and WR# strobes
	//bit4,3 CLKSPD1,0 = 8051 clock speed: 00=12, 01=24, 10=48, 11=X
	//bit2	 CLKINV=1 to invert CLKOUT signal
	//bit1	 CLKOE=1 to drive CLKOUT pin
	//bit0	 8051RES=1 to reset 8051
	// Want: 0001 0010	<-- 48MHz, output enabled.
	//		 0000 0010	<-- 12MHz, output enabled.

//	CPUCS = 0x12;  // 48MHz, output to CLKOUT signal enabled.
//	CPUCS = 0x0a;  // 24MHz, output to CLKOUT signal enabled.
	CPUCS = 0x02;  // 12MHz, output to CLKOUT signal enabled.

	// Okay, the most important config register: (default: 10000000)
	//	bit7:1 		1 = internal clocking of IFCLK; 0 = external
	//	bit6:1 		0 = 30MHz; 1 = 48MHz
	//	bit5:1 		1 = enable output to IFCLK
	//	bit4:0 		1 = invert IFCLK
	//	bit3:1 		1 = async mode; 0 = sync
	//	bit2:0 		1 = drive GSTATE[0:2] on PORTE[0:2] (irrelevant for 56-pin package)
	//	bit1,0:11	00: ports; 01: reserved; 10: GPIF; 11: Slave FIFO (ext master),
	IFCONFIG = 0xeb;
	SYNCDELAY;

	REVCTL = 0x03;	 // See TRM...
	SYNCDELAY;

		// Configure EP6 (IN):
		// EP6CFG:
		//	1 bit7: VALID: 1 = enable
		//	1 bit6: DIR:   1 = in; 0 = out
		// 10 bit5,4: TYPE1,0: 00 = ivalid; 01=isochronous; 10=bulk; 11=interrupt
		//	0 bit3: SIZE: 0 = 512 bytes; 1 = 1024 bytes
		//	0 bit2: 0
		// 00 bit1,0: BUF1,0: 00=quad; 01=<inval>; 10=double; 11=triple
		// Want: 1110 0010 (enabled, IN, BULK, double-buffered 512 bytes)
		// Configure EP6 (IN) for bulk input, quad-buffered (4*512 bytes).
	EP6CFG = 0xe0;

	SYNCDELAY;

	FIFORESET = 0x80;  SYNCDELAY;  // NAK all requests from host.
	FIFORESET = 0x02;  SYNCDELAY;  // Reset individual EP (2,4,6,8)
	FIFORESET = 0x04;  SYNCDELAY;
	FIFORESET = 0x06;  SYNCDELAY;
	FIFORESET = 0x08;  SYNCDELAY;
	FIFORESET = 0x00;  SYNCDELAY;  // Resume normal operation.

	// Configure EP6
	//0 b4:AOUT
	//1 b3:AIN
	//1 b2:ZERO
	//0 b1:0
	//1 b0:WIDE16bit
	EP6FIFOCFG = 0x0d;
	SYNCDELAY;

	// PORTACFG: FLAGD SLCS(*) 0 0 0 0 INT1 INT0
	PORTACFG = 0x00;
	SYNCDELAY; // maybe not needed

	// All default polarities: SLWR active low,...
	FIFOPINPOLAR=0x00;
	SYNCDELAY;

	// This determines how much data is accumulated in the FIFOs before a
	// USB packet is committed. Use 512 bytes to be sure.
	EP6AUTOINLENH = 0x02; // MSB
	SYNCDELAY;
	EP6AUTOINLENL = 0x00; // LSB
	SYNCDELAY;
}


void main()
{
	Initialize();

	for(;;)
	{
		// Actually, there's nothing to do for the '8051 CPU while all
		// the data is flowing inwards.
	}
}
